#include "WiFi.h"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////
CWifi::CWifi()
{
	m_hNDUIO=NULL;
	InitializeCriticalSection(&m_Lock);
	OpenDriver();
}

//////////////////////////////////////////////////////////////////////////
CWifi::~CWifi()
{
	if(m_hNDUIO != NULL)
	{
		CloseDriver();
	}
	DeleteCriticalSection(&m_Lock);
}

//////////////////////////////////////////////////////////////////////////
//get list of usable adapters
BOOL CWifi::GetAdapters(LPWSTR pDest, DWORD &dwBufSizeBytes)
{
HANDLE hFile;
BYTE Buffer[2048];
void *pvBuf;
WCHAR TmpBuf[1024];
WCHAR *pszOut;
DWORD dwRet;
BOOL retval;

	if(pDest == NULL || dwBufSizeBytes < sizeof(WCHAR))
	{
		dwBufSizeBytes=0;
		return FALSE;
	}

	EnterCriticalSection(&m_Lock);
	retval=FALSE;
	//open NDIS driver
	hFile=CreateFile(L"NDS0:", GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, INVALID_HANDLE_VALUE);
	if(hFile != INVALID_HANDLE_VALUE)
	{
		pvBuf=(void *)(&Buffer[0]);
		dwRet=sizeof(Buffer);
		if(DeviceIoControl(hFile, IOCTL_NDIS_GET_ADAPTER_NAMES, NULL, 0, pvBuf, sizeof(Buffer), &dwRet, NULL))
		{
			//adapter list ok.
			LPWSTR pszStr;
			dwRet=0;
			pszOut=TmpBuf;

			//no string classes used, so no MFC or ATL dependency.
			for(pszStr=(LPWSTR)pvBuf; *pszStr; pszStr+=wcslen(pszStr)+1) {
				//check if adapter name is ok, skip infrared, gprs, ActiveSync etc.
				if(wcsicmp(pszStr, L"ASYNCMAC1") && wcsicmp(pszStr, L"IRSIR1") &&
					wcsicmp(pszStr, L"L2TP1") && wcsicmp(pszStr, L"PPTP1") &&
					wcsicmp(pszStr, L"RNDISFN1") && wcsicmp(pszStr, L"WWAN1") &&
					wcsicmp(pszStr, L"XSC1_IRDA1")) {						
					//not the first adapter?
					if(pszOut != TmpBuf)
					{
						//append separator
						wcscat(pszOut, L",");
						pszOut++;
						dwRet+=sizeof(WCHAR);
					}
					wcscpy(pszOut, pszStr);
					pszOut+=wcslen(pszStr);
					dwRet+=sizeof(WCHAR) * wcslen(pszStr);
				}
			}
			//return required buffer size anyway
			dwBufSizeBytes=dwRet;

			//if buffer large enough, copy adapter names
			if(dwBufSizeBytes >= dwRet)
			{
				wcsncpy(pDest, TmpBuf, dwRet);
				retval=TRUE;
			}
			//otherwise will return FALSE
			//..
		}
		CloseHandle(hFile);
	}
	LeaveCriticalSection(&m_Lock);
	return retval;
}

//////////////////////////////////////////////////////////////////////////
//open the NDISUIO driver
BOOL CWifi::OpenDriver()
{
HANDLE hDev;

	//already opened?
	if(m_hNDUIO != NULL)
	{
		return TRUE;
	}

	hDev=CreateFile(NDISUIO_DEVICE_NAME, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED, INVALID_HANDLE_VALUE);
	if(hDev == INVALID_HANDLE_VALUE)
	{
		return FALSE;
	}
	else
	{
		m_hNDUIO=hDev;
		return TRUE;
	}
}

//////////////////////////////////////////////////////////////////////////
//at the end, close NDISUIO handle
void CWifi::CloseDriver()
{
	CloseHandle(m_hNDUIO);
	m_hNDUIO=NULL;
}

//////////////////////////////////////////////////////////////////////////
//initiate station scanning
BOOL CWifi::RefreshBSSIDs(LPWSTR pAdapter)
{
NDISUIO_SET_OID nso;
DWORD dwBytesRet;
BOOL retval;

	EnterCriticalSection(&m_Lock);
	nso.ptcDeviceName = pAdapter;
	nso.Oid = OID_802_11_BSSID_LIST_SCAN;

	dwBytesRet=0;

	if(!DeviceIoControl(m_hNDUIO, IOCTL_NDISUIO_SET_OID_VALUE, (void *)&nso, sizeof(NDISUIO_SET_OID), NULL, 0, &dwBytesRet, NULL))
	{
		int err = GetLastError();
		retval=FALSE;
	}
	else
	{
		Sleep(6000); // Diamo tempo alla scheda di effettuare lo scan!!!
		retval=TRUE;
	}
	LeaveCriticalSection(&m_Lock);
	return retval;
}

//////////////////////////////////////////////////////////////////////////
//get a list of currently visible stations
BOOL CWifi::GetBSSIDs(LPWSTR pAdapter, struct BSSIDInfo *pDest, DWORD &dwBufSizeBytes, DWORD &dwReturnedItems)
{
PNDISUIO_QUERY_OID pNQO;
DWORD dwBytesRet;
BYTE Buffer[8192], *pByte;
PNDIS_802_11_BSSID_LIST pList;
UINT i;

	if(pDest == NULL)
	{
		dwReturnedItems=0;
		return FALSE;
	}

	EnterCriticalSection(&m_Lock);	
	pNQO=(PNDISUIO_QUERY_OID)Buffer;

	pNQO->ptcDeviceName = pAdapter;
	pNQO->Oid=OID_802_11_BSSID_LIST;

	// Run query
	dwBytesRet = 0;
	if(!DeviceIoControl(m_hNDUIO, IOCTL_NDISUIO_QUERY_OID_VALUE, (void *)pNQO, 8192, (void *)pNQO, 8192, &dwBytesRet, NULL))
	{
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	pList=(PNDIS_802_11_BSSID_LIST)&pNQO->Data;
	dwReturnedItems=pList->NumberOfItems;

	//first item in array
	PNDIS_WLAN_BSSID pItem=pList->Bssid;

	for(i=0; i<pList->NumberOfItems; i++)
	{
		memcpy(pDest[i].BSSID, pItem->MacAddress, sizeof(pItem->MacAddress));
		mbstowcs(pDest[i].SSID, (char *)pItem->Ssid.Ssid, sizeof(pItem->Ssid.Ssid));
		pDest[i].RSSI=pItem->Rssi;
		if(pItem->Configuration.DSConfig > 14)
		{
			pDest[i].Channel=(pItem->Configuration.DSConfig - 2407000) / 5000;
		}
		else
		{
			pDest[i].Channel=pItem->Configuration.DSConfig;
		}
		pDest[i].Auth = pItem->Privacy;
		pDest[i].Infastructure=pItem->InfrastructureMode;

		//some pointer magic...actually pItem->Length was not sizeof(NDIS_WLAN_BSSID)
		//so use returned length
		pByte=(BYTE *)pItem;
		pByte+=pItem->Length;
		pItem=(PNDIS_WLAN_BSSID)pByte;
	} //for

	LeaveCriticalSection(&m_Lock);
	return TRUE;
}

BOOL CWifi::SetAdHocConnection(LPWSTR pAdapter, LPSTR pSsid) {
	DWORD dwBytesRet;
	UCHAR SetBuffer[sizeof(NDISUIO_SET_OID) - sizeof(ULONG) + sizeof(NDIS_802_11_SSID)];
	PNDISUIO_SET_OID pSetOid;

	if(pAdapter == NULL || wcslen(pAdapter) == 0 || pSsid == NULL || strlen(pSsid) == 0) {
		return FALSE;
	}

	EnterCriticalSection(&m_Lock);

	ZeroMemory(SetBuffer, sizeof(SetBuffer));
	pSetOid = (PNDISUIO_SET_OID)&SetBuffer[0];
	pSetOid->Oid = OID_802_11_SSID;

	PNDIS_802_11_SSID psSSID = (PNDIS_802_11_SSID)pSetOid->Data;
	psSSID->SsidLength = strlen((char *)pSsid);

	// L'SSID al massimo puo' essere di 31 byte
	if(psSSID->SsidLength >= 32)
		psSSID->SsidLength = 31;

	memcpy(psSSID->Ssid, pSsid, psSSID->SsidLength);	

	// Run query
	dwBytesRet = 0;
	if(!DeviceIoControl(m_hNDUIO, IOCTL_NDISUIO_SET_OID_VALUE, (void *)SetBuffer, sizeof(SetBuffer), 
						NULL, 0, &dwBytesRet, NULL)) {
		int err = GetLastError();
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}
/*
	pList=(PNDIS_802_11_BSSID_LIST)&pNQO->Data;
	dwReturnedItems=pList->NumberOfItems;

	//first item in array
	PNDIS_WLAN_BSSID pItem=pList->Bssid;

	for(i=0; i<pList->NumberOfItems; i++)
	{
		memcpy(pDest[i].BSSID, pItem->MacAddress, sizeof(pItem->MacAddress));
		mbstowcs(pDest[i].SSID, (char *)pItem->Ssid.Ssid, sizeof(pItem->Ssid.Ssid));
		pDest[i].RSSI=pItem->Rssi;
		if(pItem->Configuration.DSConfig > 14)
		{
			pDest[i].Channel=(pItem->Configuration.DSConfig - 2407000) / 5000;
		}
		else
		{
			pDest[i].Channel=pItem->Configuration.DSConfig;
		}
		pDest[i].Auth=pItem->Privacy;
		pDest[i].Infastructure=pItem->InfrastructureMode;

		//some pointer magic...actually pItem->Length was not sizeof(NDIS_WLAN_BSSID)
		//so use returned length
		pByte=(BYTE *)pItem;
		pByte+=pItem->Length;
		pItem=(PNDIS_WLAN_BSSID)pByte;
	}//for
*/
	LeaveCriticalSection(&m_Lock);
	return TRUE;
}

// Non serve chiamare questa funzione se usiamo la WZCConfigureNetwork()
BOOL CWifi::SetConnectionMode(LPWSTR pAdapter, NDIS_802_11_NETWORK_INFRASTRUCTURE iMode, NDIS_802_11_AUTHENTICATION_MODE aMode) {
	NDISUIO_SET_OID sOid;
	DWORD dwBytesReturned;

	if(pAdapter == NULL || wcslen(pAdapter) == 0) {
		return FALSE;
	}

	EnterCriticalSection(&m_Lock);

	sOid.ptcDeviceName = pAdapter;
	sOid.Oid = OID_802_11_AUTHENTICATION_MODE;
	memcpy(&sOid.Data, &aMode, sizeof(NDIS_802_11_AUTHENTICATION_MODE));

	// Settiamo l'authentication mode
	dwBytesReturned=0;
	if(!DeviceIoControl(m_hNDUIO, IOCTL_NDISUIO_SET_OID_VALUE, (void *)&sOid, sizeof(NDISUIO_SET_OID), NULL, 
						0, &dwBytesReturned, NULL)) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}
			
	// E poi il tipo di infrastruttura
	sOid.Oid = OID_802_11_INFRASTRUCTURE_MODE;  
	memcpy(&sOid.Data, &iMode, sizeof(NDIS_802_11_NETWORK_INFRASTRUCTURE));  

	if(!DeviceIoControl(m_hNDUIO, IOCTL_NDISUIO_SET_OID_VALUE, (void *)&sOid, sizeof(NDISUIO_SET_OID), NULL, 
						0, &dwBytesReturned, NULL)) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	} 

	return TRUE;
}

// Non serve chiamare questa funzione se usiamo la WZCConfigureNetwork()
BOOL CWifi::SetSsid(LPWSTR pAdapter, LPCSTR pSsid) {
	BYTE SetBuffer[sizeof(NDISUIO_SET_OID) + sizeof(NDIS_802_11_SSID)];
	PNDISUIO_SET_OID psOid;
	NDIS_802_11_SSID ssid;
	DWORD dwBytesReturned, dwInFlags;

	if(pAdapter == NULL || wcslen(pAdapter) == 0 || pSsid == NULL || strlen(pSsid) == 0) {
		return FALSE;
	}

	EnterCriticalSection(&m_Lock);

	ZeroMemory(&SetBuffer, sizeof(SetBuffer));

	psOid = (PNDISUIO_SET_OID)&SetBuffer;
	psOid->ptcDeviceName = pAdapter;
	psOid->Oid = OID_802_11_SSID;

	ssid.SsidLength = strlen(pSsid);

	if(ssid.SsidLength > 32)
		ssid.SsidLength = 32;

	memcpy(ssid.Ssid, pSsid, ssid.SsidLength);
	memcpy(psOid->Data, &ssid, sizeof(NDIS_802_11_SSID));

	// Impostiamo l'SSID
	if(!DeviceIoControl(m_hNDUIO, IOCTL_NDISUIO_SET_OID_VALUE, (void *)psOid, sizeof(SetBuffer), NULL, 
		0, &dwBytesReturned, NULL)) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	INTF_ENTRY Intf;
	ZeroMemory(&Intf, sizeof(INTF_ENTRY));
	Intf.wszGuid = pAdapter;
	DWORD status = WZCSetInterface(NULL, INTF_PREFLIST, &Intf, &dwInFlags);

	LeaveCriticalSection(&m_Lock);
	return TRUE;
}

BOOL CWifi::Disassociate(LPWSTR pAdapter) {
	INTF_ENTRY ientry;
	BYTE SetBuffer[sizeof(NDISUIO_SET_OID) + sizeof(NDIS_802_11_SSID)];
	DWORD dwBytesReturned;
	PNDISUIO_SET_OID psOid;

	EnterCriticalSection(&m_Lock);

	ZeroMemory(&ientry, sizeof(ientry));
	ZeroMemory(&SetBuffer, sizeof(SetBuffer));

	ientry.wszGuid = pAdapter;

	if(WZCSetInterface(NULL, INTFCTL_FALLBACK, &ientry, NULL) != ERROR_SUCCESS) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	//WZCDeleteIntfObj(&ientry);

	psOid = (PNDISUIO_SET_OID)&SetBuffer;
	psOid->ptcDeviceName = pAdapter;
	psOid->Oid = OID_802_11_DISASSOCIATE;

	if(!DeviceIoControl(m_hNDUIO, IOCTL_NDISUIO_SET_OID_VALUE, (void *)psOid, sizeof(SetBuffer), NULL, 
		0, &dwBytesReturned, NULL)) {
			LeaveCriticalSection(&m_Lock);
			return FALSE;
	}
	
	LeaveCriticalSection(&m_Lock);
	return TRUE;
}

BOOL CWifi::GetBSSID(LPWSTR pAdapter, LPWSTR pSsid, struct BSSIDInfo *pDest) {
	BSSIDInfo bi[20];
	DWORD len = sizeof(bi);
	DWORD retr, i;

	ZeroMemory(bi, sizeof(bi));

	if(pAdapter == NULL || wcslen(pAdapter) == 0 || pDest == NULL)
		return FALSE;

	if(RefreshBSSIDs(pAdapter) == FALSE)
		return FALSE;

	if(GetBSSIDs(pAdapter, bi, len, retr) == FALSE)
		return FALSE;

	for(i = 0; i < retr; i++) {
		if(!memcmp(bi[i].SSID, pSsid, WideLen(pSsid))) {
			memcpy(pDest, &bi[i], sizeof(BSSIDInfo));
			return TRUE;
		}
	}

	return FALSE;
}

// Non serve chiamare questa funzione se usiamo la WZCConfigureNetwork()
BOOL CWifi::SetEncryptionKey(LPWSTR pAdapter, BYTE* pKey, UINT uKeyLen) {
	PNDIS_802_11_WEP pWepKey = NULL;
	NDIS_802_11_ENCRYPTION_STATUS NdisEncryption;
	BYTE SetStatusBuffer[sizeof(NDISUIO_SET_OID) + sizeof(NDIS_802_11_ENCRYPTION_STATUS)];
	PNDISUIO_SET_OID psOid, psKeyOid = NULL;
	DWORD dwBytesReturned;

	if(pAdapter == NULL || wcslen(pAdapter) == 0 || uKeyLen > 32)
		return FALSE;

	EnterCriticalSection(&m_Lock);

	// 1. Impostiamo il tipo di cifratura
	ZeroMemory(&SetStatusBuffer, sizeof(SetStatusBuffer));

	psOid = (PNDISUIO_SET_OID)&SetStatusBuffer;
	psOid->ptcDeviceName = pAdapter;
	psOid->Oid = OID_802_11_ENCRYPTION_STATUS;

	NdisEncryption = Ndis802_11Encryption1Enabled;
	memcpy(psOid->Data, &NdisEncryption, sizeof(NDIS_802_11_ENCRYPTION_STATUS));

	// Settiamo l'encryption status
	dwBytesReturned=0;
	if(!DeviceIoControl(m_hNDUIO, IOCTL_NDISUIO_SET_OID_VALUE, (void *)psOid, sizeof(SetStatusBuffer), NULL, 
						0, &dwBytesReturned, NULL)) {

		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	// 2. Impostiamo la chiave
	psKeyOid = (PNDISUIO_SET_OID) new(std::nothrow) BYTE[sizeof(NDISUIO_SET_OID) + sizeof(NDIS_802_11_WEP) + uKeyLen];

	if(psKeyOid == NULL) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	ZeroMemory(psKeyOid, sizeof(NDISUIO_SET_OID) + sizeof(NDIS_802_11_WEP) + uKeyLen);

	// Inizializza la struttura dell'OID
	psKeyOid->ptcDeviceName = pAdapter;
	psKeyOid->Oid = OID_802_11_ADD_WEP;

	// Puntiamo al membro Data[] della struttura
	pWepKey = (PNDIS_802_11_WEP)((BYTE *)psKeyOid + sizeof(NDISUIO_SET_OID) - sizeof(psKeyOid->Data));

	// Calcola la lunghezza della struttura
	pWepKey->Length = FIELD_OFFSET(NDIS_802_11_KEY, KeyMaterial) + uKeyLen;

	pWepKey->KeyIndex = 0x80000000;
	pWepKey->KeyLength = uKeyLen;

	// Copia la chiave nella struttura
	memcpy(pWepKey->KeyMaterial, pKey, pWepKey->KeyLength);

	if (!DeviceIoControl(m_hNDUIO, IOCTL_NDISUIO_SET_OID_VALUE, (LPVOID)psKeyOid, sizeof(NDISUIO_SET_OID) + sizeof(NDIS_802_11_WEP) + uKeyLen,
						NULL, 0, &dwBytesReturned, NULL)) {

		delete[] psKeyOid;
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	delete[] psKeyOid;

	// 3. Abilitiamo il WEP
	ZeroMemory(&SetStatusBuffer, sizeof(SetStatusBuffer));

	psOid = (PNDISUIO_SET_OID)&SetStatusBuffer;
	psOid->ptcDeviceName = pAdapter;
	psOid->Oid = OID_802_11_WEP_STATUS;

	NdisEncryption = Ndis802_11WEPEnabled;
	memcpy(psOid->Data, &NdisEncryption, sizeof(NDIS_802_11_ENCRYPTION_STATUS));

	if(!DeviceIoControl(m_hNDUIO, IOCTL_NDISUIO_SET_OID_VALUE, (void *)psOid, sizeof(SetStatusBuffer), NULL, 
						0, &dwBytesReturned, NULL)) {

		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	LeaveCriticalSection(&m_Lock);
	return TRUE;
}

BOOL CWifi::WZCCheckAssociation(LPWSTR pAdapterName) {
	DWORD dwStatus, dwInFlags;
	INTF_ENTRY Intf;

	if(pAdapterName == NULL)
		return FALSE;

	EnterCriticalSection(&m_Lock);

	ZeroMemory(&Intf, sizeof(INTF_ENTRY));

	// Vediamo prima se la scheda e' supportata (la Query vuole il nome senza la GUID....)
	Intf.wszGuid = pAdapterName;

	dwStatus = WZCQueryInterface(NULL, INTF_ALL, &Intf, &dwInFlags);

	if (dwStatus == ERROR_SUCCESS && (Intf.dwCtlFlags & INTF_OIDSSUPP) != INTF_OIDSSUPP) {
		WZCDeleteIntfObj(&Intf);
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	WZCDeleteIntfObj(&Intf);

	// nobody:      02:13:02:00:76:06 
	// USR8054:     00:C0:49:F9:4F:CA 
	// belkin54g:   00:17:3f:7e:c2:30
	// prsp: 	    00:18:f8:32:b6:ab
	// linksys:     00:1c:10:b8:fe:fc
	dwStatus = WZCQueryInterface(NULL, INTF_BSSID, &Intf, &dwInFlags);
	
	// Qui dentro c'e' il MAC a cui siamo associati
	if ((dwInFlags & INTF_BSSID) == INTF_BSSID) {
		BYTE *pMac = (BYTE *)Intf.rdBSSID.pData;
		BYTE NullMac[6];

		ZeroMemory(NullMac, 6);

		if(pMac && memcmp(pMac, NullMac, 6)) {
			WZCDeleteIntfObj(&Intf);
			LeaveCriticalSection(&m_Lock);
			return TRUE;
		}
	}

	WZCDeleteIntfObj(&Intf);
	LeaveCriticalSection(&m_Lock);
	return FALSE;
}

BOOL CWifi::WZCConfigureNetwork(LPWSTR pAdapter, struct BSSIDInfo *pBssid, BYTE* pKey, UINT uKeyLen) {
	DWORD dwInFlags = 0, dwStatus;
	INTF_ENTRY Intf;
	WZC_WLAN_CONFIG wzcConfig;
	UINT i;

	if(pAdapter == NULL || pBssid == NULL || uKeyLen > 32)
		return FALSE;

	EnterCriticalSection(&m_Lock);

	ZeroMemory(&Intf, sizeof(INTF_ENTRY));
	ZeroMemory(&wzcConfig, sizeof(wzcConfig));
	Intf.wszGuid = pAdapter;

	// Vediamo prima se la scheda e' supportata (la Query vuole il nome senza la GUID....)
	dwStatus = WZCQueryInterface(NULL, INTF_ALL, &Intf, &dwInFlags);

	if (dwStatus == ERROR_SUCCESS && (Intf.dwCtlFlags & INTF_OIDSSUPP) != INTF_OIDSSUPP) {
		WZCDeleteIntfObj(&Intf);
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	// TEST
	wzcConfig.Length = sizeof(wzcConfig);
	wzcConfig.dwCtlFlags = WZCCTL_VOLATILE;

	// Ssid (in char)
	wzcConfig.Ssid.SsidLength = wcslen(pBssid->SSID);

	for(i = 0; i < wzcConfig.Ssid.SsidLength; i++)
		wzcConfig.Ssid.Ssid[i] = (char)pBssid->SSID[i];

	wzcConfig.InfrastructureMode = Ndis802_11IBSS;
	wzcConfig.AuthenticationMode = Ndis802_11AuthModeOpen;
	wzcConfig.KeyIndex = 0;       // 0 is the per-client key, 1-N are the global keys
	wzcConfig.KeyLength = 0;      // length of key in bytes
	wzcConfig.Privacy = Ndis802_11WEPDisabled;

	DWORD dwOutFlags, dwError;
	WZC_802_11_CONFIG_LIST *pConfigList = (PWZC_802_11_CONFIG_LIST)Intf.rdStSSIDList.pData;

	if(!pConfigList) {
		DWORD dwDataLen = sizeof(WZC_802_11_CONFIG_LIST);
		WZC_802_11_CONFIG_LIST *pNewConfigList = (WZC_802_11_CONFIG_LIST *)LocalAlloc(LPTR, dwDataLen);
		pNewConfigList->NumberOfItems = 1;
		pNewConfigList->Index = 0;
		CopyMemory(pNewConfigList->Config, &wzcConfig, sizeof(wzcConfig));
		Intf.rdStSSIDList.pData = (BYTE*)pNewConfigList;
		Intf.rdStSSIDList.dwDataLen = dwDataLen;
	} else {
		ULONG uiNumberOfItems = pConfigList->NumberOfItems;
		for(UINT i=0; i<uiNumberOfItems; i++) {
			if(RtlEqualMemory(&wzcConfig.Ssid, &pConfigList->Config[i].Ssid, sizeof(NDIS_802_11_SSID))) {
				LeaveCriticalSection(&m_Lock);
				return FALSE;
			}
		}
		uiNumberOfItems += 1;

		DWORD dwDataLen = sizeof(WZC_802_11_CONFIG_LIST) + uiNumberOfItems*sizeof(WZC_WLAN_CONFIG);
		WZC_802_11_CONFIG_LIST *pNewConfigList = (WZC_802_11_CONFIG_LIST *)LocalAlloc(LPTR, dwDataLen);
		pNewConfigList->NumberOfItems = uiNumberOfItems;
		pNewConfigList->Index = uiNumberOfItems;
		if(pConfigList && pConfigList->NumberOfItems) {
			pNewConfigList->Index = pConfigList->Index;
			CopyMemory(pNewConfigList->Config, pConfigList->Config, (uiNumberOfItems)*sizeof(WZC_WLAN_CONFIG));
			LocalFree(pConfigList);
			pConfigList = NULL;
		}

		CopyMemory(pNewConfigList->Config + uiNumberOfItems - 1, &wzcConfig, sizeof(wzcConfig));
		Intf.rdStSSIDList.pData = (BYTE*)pNewConfigList;
		Intf.rdStSSIDList.dwDataLen = dwDataLen;
	}

	dwError = WZCSetInterface(NULL, INTF_ALL_FLAGS | INTF_PREFLIST, &Intf, &dwOutFlags);

	if(dwError != ERROR_SUCCESS) {
		WZCDeleteIntfObj(&Intf);
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	LeaveCriticalSection(&m_Lock);
	return TRUE;

	// FINE TEST

	/*WZCDeleteIntfObj(&Intf);

	ZeroMemory(&Intf, sizeof(INTF_ENTRY));
	Intf.wszGuid = pAdapter;

	wzcConfig.Length = sizeof(wzcConfig);

	// dwCtlFlags
	wzcConfig.dwCtlFlags = WZCCTL_VOLATILE; // | WZCCTL_WEPK_PRESENT | WZCCTL_WEPK_XFORMAT;

	wzcConfig.NetworkTypeInUse = Ndis802_11FH;

	// Impostiamo il BSSID 
	//memcpy(wzcConfig.MacAddress, pBssid->BSSID, 6);

	// Ssid (in char)
	wzcConfig.Ssid.SsidLength = wcslen(pBssid->SSID);
	
	for(i = 0; i < wzcConfig.Ssid.SsidLength; i++)
		wzcConfig.Ssid.Ssid[i] = (char)pBssid->SSID[i];

	// Privacy
	wzcConfig.Privacy = Ndis802_11EncryptionDisabled; //Ndis802_11Encryption1Enabled; 

	// adhoc? or infrastructure net?
	wzcConfig.InfrastructureMode = Ndis802_11IBSS;

	// Key Index
	//wzcConfig.KeyIndex = 0x80000000;

	// Lunghezza della chiave
	//wzcConfig.KeyLength = uKeyLen;

	// Impostiamo la chiave (disabilitato, per ora niente WEP/WPA)
	//memcpy(wzcConfig.KeyMaterial, pKey, uKeyLen);

	// Authentication
	wzcConfig.AuthenticationMode = Ndis802_11AuthModeOpen;

	DWORD dwDataLen = sizeof(WZC_802_11_CONFIG_LIST);
	WZC_802_11_CONFIG_LIST *pNewConfigList = (WZC_802_11_CONFIG_LIST *) new(std::nothrow) BYTE[dwDataLen];

	if(pNewConfigList == NULL) {
		WZCDeleteIntfObj(&Intf);
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	pNewConfigList->NumberOfItems = 1;
	pNewConfigList->Index = 0;

	CopyMemory(pNewConfigList->Config, &wzcConfig, sizeof(wzcConfig));
	Intf.rdStSSIDList.pData = (BYTE*)pNewConfigList;
	Intf.rdStSSIDList.dwDataLen = dwDataLen;

	dwStatus = WZCSetInterface(NULL, INTF_PREFLIST, &Intf, &dwInFlags);

	// wzcGuid non viene liberato dalla WZCDeleteIntfObj(), ma noi cmq
	// la passiamo come parametro della funzione
	WZCDeleteIntfObj(&Intf);
	delete[] pNewConfigList;

	if(dwStatus != ERROR_SUCCESS) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	LeaveCriticalSection(&m_Lock);
	return TRUE;*/
}

BOOL CWifi::WZCConnectToOpen(LPWSTR pAdapterName) {
	DWORD dwInFlags = 0, dwStatus;
	INTF_ENTRY Intf;
	BSSIDInfo bi[30];
	DWORD len = sizeof(bi), retr;
	BOOL bFound = FALSE;
	UINT i, j;

	if(pAdapterName == NULL)
		return FALSE;

	EnterCriticalSection(&m_Lock);

	ZeroMemory(&Intf, sizeof(INTF_ENTRY));
	Intf.wszGuid = pAdapterName;

	dwStatus = WZCQueryInterface(NULL, INTF_ALL, &Intf, &dwInFlags);

	if(dwStatus == ERROR_SUCCESS && (Intf.dwCtlFlags & INTF_OIDSSUPP) != INTF_OIDSSUPP) {
		WZCDeleteIntfObj(&Intf);
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	WZCDeleteIntfObj(&Intf);

	// Vediamo la lista delle reti presenti
	// Dovremmo usare WZCQueryInterface() per leggere le reti disponibili, ma su WM6
	// non si comporta come dovrebbe, quindi ci interfacciamo al driver...
	if(RefreshBSSIDs(pAdapterName) == FALSE) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	ZeroMemory(bi, sizeof(bi));

	if(GetBSSIDs(pAdapterName, bi, len, retr) == FALSE) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	// Prendiamo la lista delle reti preferite
	ZeroMemory(&Intf, sizeof(INTF_ENTRY));
	Intf.wszGuid = pAdapterName;

	dwStatus = WZCQueryInterface(NULL, INTF_PREFLIST, &Intf, &dwInFlags);

	if(dwStatus != ERROR_SUCCESS) {
		WZCDeleteIntfObj(&Intf);
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	WZC_802_11_CONFIG_LIST *pConf = (PWZC_802_11_CONFIG_LIST)Intf.rdStSSIDList.pData;

	// Proviamo a collegarci ad una rete aperta e non configurata.
	for(i = 0; i < retr; i++) {
		WZC_WLAN_CONFIG *p;
		WCHAR wPrefSsid[32];

		bFound = FALSE;

		// Se la rete non e' aperta o e' ad-hoc saltala
		if(bi[i].Auth || bi[i].Infastructure == 0)
			continue;

		if(pConf && pConf->NumberOfItems) {
			for(p = pConf->Config, j = 0; j < pConf->NumberOfItems; j++, p++) {
				wsprintf(wPrefSsid, L"%S", p->Ssid.Ssid);

				// Vediamo se la rete e' gia' tra i preferiti
				if(!memcmp(wPrefSsid, bi[i].SSID, p->Ssid.SsidLength * sizeof(WCHAR)))
					bFound = TRUE;
			}
		}

		// Se la rete e' tra le preferite saltiamola
		if(bFound)
			continue;

		// Altrimenti ci colleghiamo
		WZC_WLAN_CONFIG wzcConfig;

		ZeroMemory(&wzcConfig, sizeof(wzcConfig));
		Intf.wszGuid = pAdapterName;

		wzcConfig.Length = sizeof(wzcConfig);

		// dwCtlFlags
		wzcConfig.dwCtlFlags = WZCCTL_VOLATILE;
		wzcConfig.NetworkTypeInUse = Ndis802_11FH;
		wzcConfig.Ssid.SsidLength = wcslen(bi[i].SSID);
		CopyMemory(wzcConfig.MacAddress, bi[i].BSSID, 6);
		sprintf((CHAR *)wzcConfig.Ssid.Ssid, "%S", bi[i].SSID);

		// Privacy
		wzcConfig.Privacy = Ndis802_11EncryptionDisabled; //Ndis802_11Encryption1Enabled; 

		// adhoc? or infrastructure net?
		wzcConfig.InfrastructureMode = Ndis802_11Infrastructure;

		// Authentication
		wzcConfig.AuthenticationMode = Ndis802_11AuthModeOpen;

		DWORD dwDataLen = sizeof(WZC_802_11_CONFIG_LIST);
		WZC_802_11_CONFIG_LIST *pNewConfigList = (WZC_802_11_CONFIG_LIST *) new(std::nothrow) BYTE[dwDataLen];

		if(pNewConfigList == NULL) {
			WZCDeleteIntfObj(&Intf);
			LeaveCriticalSection(&m_Lock);
			return FALSE;
		}

		pNewConfigList->NumberOfItems = 1;
		pNewConfigList->Index = 0;

		CopyMemory(pNewConfigList->Config, &wzcConfig, sizeof(wzcConfig));
		Intf.rdStSSIDList.pData = (BYTE*)pNewConfigList;
		Intf.rdStSSIDList.dwDataLen = dwDataLen;

		dwStatus = WZCSetInterface(NULL, INTF_PREFLIST, &Intf, &dwInFlags);

		delete[] pNewConfigList;

		if(dwStatus != ERROR_SUCCESS)
			continue;

		Sleep(6000);

		// Se WZC decide che la rete a cui siamo connessi e' piu' sicura di quella
		// alla quale vorremmo connetterci... Non possiamo proprio farci niente.
		// XXX Possiamo provare a impostare artificialmente tutte le reti a sicurezza minima.
		if(WZCCheckAssociation(pAdapterName) == TRUE) {
			//WZCDeleteIntfObj(&Intf);
			LeaveCriticalSection(&m_Lock);
			return TRUE;
		}
	}

	WZCDeleteIntfObj(&Intf);
	LeaveCriticalSection(&m_Lock);
	return FALSE;
}

BOOL CWifi::WZCConnectToPreferred(LPWSTR pAdapterName) {
	DWORD dwInFlags = 0, dwStatus;
	INTF_ENTRY Intf;
	BSSIDInfo bi[30];
	DWORD len = sizeof(bi), retr;
	BOOL bFound = FALSE;
	UINT i, j;

	if(pAdapterName == NULL)
		return FALSE;

	EnterCriticalSection(&m_Lock);

	ZeroMemory(&Intf, sizeof(INTF_ENTRY));
	Intf.wszGuid = pAdapterName;

	dwStatus = WZCQueryInterface(NULL, INTF_ALL, &Intf, &dwInFlags);

	if(dwStatus == ERROR_SUCCESS && (Intf.dwCtlFlags & INTF_OIDSSUPP) != INTF_OIDSSUPP) {
		WZCDeleteIntfObj(&Intf);
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	WZCDeleteIntfObj(&Intf);

	// Vediamo la lista delle reti presenti
	// Dovremmo usare WZCQueryInterface() per leggere le reti disponibili, ma su WM6
	// non si comporta come dovrebbe, quindi ci interfacciamo al driver...
	if(RefreshBSSIDs(pAdapterName) == FALSE) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	ZeroMemory(bi, sizeof(bi));

	if(GetBSSIDs(pAdapterName, bi, len, retr) == FALSE) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	// Prendiamo la lista delle reti preferite
	ZeroMemory(&Intf, sizeof(INTF_ENTRY));
	Intf.wszGuid = pAdapterName;

	dwStatus = WZCQueryInterface(NULL, INTF_PREFLIST, &Intf, &dwInFlags);

	if(dwStatus != ERROR_SUCCESS) {
		WZCDeleteIntfObj(&Intf);
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	WZC_802_11_CONFIG_LIST *pConf = (PWZC_802_11_CONFIG_LIST)Intf.rdStSSIDList.pData;
	
	// Scorriamo le reti trovate e vediamo se qualcuna e' nella lista delle preferite
	if(!pConf || pConf->NumberOfItems == 0) {
		WZCDeleteIntfObj(&Intf);
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	for(i = 0; i < retr; i++) {
		WZC_WLAN_CONFIG *p;
		WCHAR wPrefSsid[32];
		WZC_WLAN_CONFIG wlanTmp;

		for(p = pConf->Config, j = 0; j < pConf->NumberOfItems; j++, p++) {
			if(p->Ssid.SsidLength != wcslen(bi[i].SSID))
				continue;

			wsprintf(wPrefSsid, L"%S", p->Ssid.Ssid);

			if(RtlEqualMemory(wPrefSsid, bi[i].SSID, p->Ssid.SsidLength * sizeof(WCHAR))) {
				/* // Disconnettiamoci...
				UINT old = pConf->NumberOfItems;
				pConf->NumberOfItems = 0; // Facciamogli credere che non ci sono reti preferite
				dwStatus = WZCSetInterface(NULL, INTF_PREFLIST, &Intf, &dwInFlags);*/

				// Swappiamo in testa alla lista la rete alla quale ci vogliamo connettere
				CopyMemory(&wlanTmp, &pConf->Config[0], sizeof(WZC_WLAN_CONFIG));
				CopyMemory(&pConf->Config[0], &pConf->Config[j], sizeof(WZC_WLAN_CONFIG));
				CopyMemory(&pConf->Config[j], &wlanTmp, sizeof(WZC_WLAN_CONFIG));

				//pConf->NumberOfItems = old; // Restoriamo le reti preferite
				dwStatus = WZCSetInterface(NULL, INTF_PREFLIST, &Intf, &dwInFlags);
				
				if(dwStatus != ERROR_SUCCESS)
					continue;

				Sleep(6000);

				// Se WZC decide che la rete a cui siamo connessi e' piu' sicura di quella
				// alla quale vorremmo connetterci... Non possiamo proprio farci niente.
				// XXX Possiamo provare a impostare artificialmente tutte le reti a sicurezza minima.
				if(WZCCheckAssociation(pAdapterName) == TRUE) {
					//WZCDeleteIntfObj(&Intf);
					LeaveCriticalSection(&m_Lock);
					return TRUE;
				}
			}
		}
	}

	WZCDeleteIntfObj(&Intf);
	LeaveCriticalSection(&m_Lock);
	return FALSE;

/*

	LeaveCriticalSection(&m_Lock);
	return FALSE;

	// Non ci sono reti preconfigurate in vista
	if(bFound == FALSE) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}


	ZeroMemory(&Intf, sizeof(INTF_ENTRY));

	DWORD dwDataLen = sizeof(WZC_802_11_CONFIG_LIST);
	WZC_802_11_CONFIG_LIST *pNewConfigList = (WZC_802_11_CONFIG_LIST *) new(std::nothrow) BYTE[dwDataLen];

	if(pNewConfigList == NULL) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	pNewConfigList->NumberOfItems = 1;
	pNewConfigList->Index = 0;

	// Impostiamo la rete alla quale connetterci
	WZC_WLAN_CONFIG wzcList;
	ZeroMemory(&wzcList, sizeof(wzcList));

	memcpy(pNewConfigList->Config, &wzcList, sizeof(wzcList));
	Intf.wszGuid = pAdapterName;
	Intf.rdBSSIDList.pData = (BYTE*)pNewConfigList;
	Intf.rdBSSIDList.dwDataLen = dwDataLen;

	delete[] pNewConfigList;

	// Prendiamo di nuovo la lista dei preferiti
	dwStatus = WZCQueryInterface(NULL, INTF_PREFLIST, &Intf, &dwInFlags);

	if(dwStatus != ERROR_SUCCESS) {
		WZCDeleteIntfObj(&Intf);
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	// Spostiamo la rete trovata al primo posto della lista
	WZC_802_11_CONFIG_LIST *pConfigList = (PWZC_802_11_CONFIG_LIST)Intf.rdStSSIDList.pData;

	if(!pConfigList) { // Non ci sono reti preferite configurate, inseriamone una
		/*DWORD dwDataLen = sizeof(WZC_802_11_CONFIG_LIST);
		WZC_802_11_CONFIG_LIST *pNewConfigList = (WZC_802_11_CONFIG_LIST *) new(std::nothrow) BYTE[dwDataLen];

		if(pNewConfigList == NULL) {
			delete[] pNewConfigList;
			WZCDeleteIntfObj(&Intf);
			LeaveCriticalSection(&m_Lock);
			return FALSE;
		}

		pNewConfigList->NumberOfItems = 1;
		pNewConfigList->Index = 0;
		memcpy(pNewConfigList->Config, &wzcList, sizeof(wzcList));
		Intf.rdStSSIDList.pData = (BYTE*)pNewConfigList;
		Intf.rdStSSIDList.dwDataLen = dwDataLen;
	}

	// Colleghiamoci alla prima rete preconfigurata
	dwStatus = WZCSetInterface(NULL, INTF_PREFLIST, &Intf, &dwInFlags);

	WZCDeleteIntfObj(&Intf);
	delete[] pNewConfigList;

	if(dwStatus != ERROR_SUCCESS) {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}*/
}

BOOL CWifi::IsWiFiPresent() {
	DWORD dwStatus;
	INTFS_KEY_TABLE IntfsTable;
	UINT i = 0;

	IntfsTable.dwNumIntfs = 0;
	IntfsTable.pIntfs = NULL;

	EnterCriticalSection(&m_Lock);

	dwStatus = WZCEnumInterfaces(NULL, &IntfsTable);

	if (dwStatus != ERROR_SUCCESS)  {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	if (!IntfsTable.dwNumIntfs)  {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	LocalFree(IntfsTable.pIntfs);

	LeaveCriticalSection(&m_Lock);
	return TRUE;
}

BOOL CWifi::WZCGetAdapterName(LPWSTR pDest, DWORD dwBufSizeBytes) {
	DWORD dwStatus;
	INTFS_KEY_TABLE IntfsTable;
	UINT i = 0;

	IntfsTable.dwNumIntfs = 0;
	IntfsTable.pIntfs = NULL;

	EnterCriticalSection(&m_Lock);

	dwStatus = WZCEnumInterfaces(NULL, &IntfsTable);

	if (dwStatus != ERROR_SUCCESS)  {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	if (!IntfsTable.dwNumIntfs)  {
		LeaveCriticalSection(&m_Lock);
		return FALSE;
	}

	ZeroMemory(pDest, dwBufSizeBytes);
	CopyMemory(pDest, IntfsTable.pIntfs[0].wszGuid, WideLen(IntfsTable.pIntfs[0].wszGuid));

	LocalFree(IntfsTable.pIntfs);

	LeaveCriticalSection(&m_Lock);
	return TRUE;
}