#define INITGUID
#include <windows.h>
#include <cemapi.h>
#include "MapiRule.h"
#include "IPCQueue.h"
#include "IPCMsg.h"

// IpcQueueLocalSA

CMailRuleClient::CMailRuleClient()
: m_qRead(NULL), m_qWrite(NULL)
{
}

CMailRuleClient::~CMailRuleClient()
{
  if (m_qRead)
    delete m_qRead;

  if (m_qWrite)
    delete m_qWrite;
}

/**
 *	QueryInterface
 **/
HRESULT CMailRuleClient::QueryInterface(REFIID rif, void** ppobj)
{
	if (!ppobj)
		return E_INVALIDARG;

	*ppobj = NULL;

	if ((rif == IID_IUnknown) || (rif == IID_IMailRuleClient)) {
		*ppobj = (LPVOID) this;
	}

	if (*ppobj)
		((LPUNKNOWN)*ppobj)->AddRef();

	return (*ppobj) ? S_OK : E_NOINTERFACE;
}

ULONG CMailRuleClient::AddRef()
{
	return InterlockedIncrement(&m_cRef);
}

ULONG CMailRuleClient::Release()
{
	InterlockedDecrement(&m_cRef);

	int nLocal = m_cRef;

	if (!m_cRef)
		delete this;

	return nLocal;
}

#define CLSID_KEY TEXT("\\CLSID\\{DD69A982-6C70-4a55-8BE4-6B32A1F9A527}")
HRESULT CMailRuleClient::Initialize(IMsgStore *pMsgStore, MRCACCESS *pmaDesidered)
{
  // MessageBox(NULL, L"Initializing queue", L"DEBUG", MB_OK);

	*pmaDesidered = MRC_ACCESS_WRITE;
  
	HKEY hKey;
  
	memset(this->m_wszServiceProvider, 0, sizeof(m_wszServiceProvider));
	m_cInitialized = false;
  
	if (RegOpenKeyEx(HKEY_CLASSES_ROOT, TEXT("\\CLSID\\{DD69A982-6C70-4a55-8BE4-6B32A1F9A527}"), 0, 0, &hKey) == ERROR_SUCCESS) {
		DWORD dwType, cData = sizeof(this->m_wszServiceProvider) * 2;

		RegQueryValueEx(hKey, TEXT("Service Provider"), 0, &dwType, (LPBYTE)this->m_wszServiceProvider, &cData);

		RegCloseKey(hKey);
		m_cInitialized = true;
	}
  
  UINT i = 0;
  while ( i < 5 )
  {
    m_qRead = new IPCQueue(L"IpcQueueLocalSAW", true);
    if (m_qRead == NULL)
    {
      //MessageBox(NULL, L"Cannot create Write queue. Retrying in 2 secs", L"DEBUG", MB_OK);
      Sleep(2000);
    }
    else
      break;
  }

  while ( i < 5 )
  { 
    m_qWrite = new IPCQueue(L"IpcQueueLocalSAR", false);
    if (m_qWrite == NULL)
    {
     // MessageBox(NULL, L"Cannot create Read queue. Retrying in 2 secs", L"DEBUG", MB_OK);
      Sleep(2000);
    }
    else
      break;
  }
  
  if (m_qRead->Open() == false)
  {
    //MessageBox(NULL, L"Cannot open Read queue.", L"DEBUG", MB_OK);
    return E_FAIL;
  }

  if (m_qWrite->Open() == false)
  {
    //MessageBox(NULL, L"Cannot open Write queue.", L"DEBUG", MB_OK);
    return E_FAIL;
  }
  
  // MessageBox(NULL, L"Queue inited.", L"DEBUG", MB_OK);
  
	return S_OK;
}

HRESULT CMailRuleClient::DeleteMessage(
		IMsgStore *pMsgStore, 
		IMessage *pMsg, 
		ULONG cbMsg, 
		LPENTRYID lpMsg, 
		ULONG cbDestFolder, 
		LPENTRYID lpDestFolder, 
		ULONG *pulEventType, 
		MRCHANDLED *pHandled)
{
	HRESULT hr = S_OK;
	ENTRYLIST lst;
	SBinary sbin;
	IMAPIFolder *pFolder = NULL;
  
	hr = pMsgStore->OpenEntry(cbDestFolder, lpDestFolder, NULL, 0, NULL, (LPUNKNOWN *) &pFolder);
  
	if (FAILED(hr))
		return hr;

	lst.cValues = 1;
	sbin.cb = cbMsg;
	sbin.lpb = (LPBYTE) lpMsg;
	lst.lpbin = &sbin;

	hr = pFolder->DeleteMessages(&lst, NULL, NULL, 0);

	if (!FAILED(hr)) {
		*pulEventType = fnevObjectDeleted;
		*pHandled = MRC_HANDLED_DONTCONTINUE;
	}

	if (pFolder)
		pFolder->Release();

	return hr;
}

HRESULT CMailRuleClient::ProcessMessage(
		IMsgStore *pMsgStore, 
		ULONG cbMsg, 
		LPENTRYID lpMsg,  
		ULONG cbDestFolder, 
		LPENTRYID lpDestFolder, 
		ULONG *pulEventType, 
		MRCHANDLED *pHandled)
{
	HRESULT hr = S_OK;
	IMessage *pMsg = NULL;
	SizedSPropTagArray(1, sptaSubject) = { 1, PR_SUBJECT };
	SizedSPropTagArray(1, sptaSenderEmailAddress) = { 1, PR_SENDER_EMAIL_ADDRESS };
	ULONG cValues = 0;
	SPropValue *pspvSubject = NULL;
	SPropValue *pspvSenderEmailAddress = NULL;

	if (this->m_cInitialized == false)
		return hr;

  //
  // open message
	//
  hr = pMsgStore->OpenEntry(cbMsg, lpMsg, NULL, 0, NULL, (LPUNKNOWN *) &pMsg);

	if (FAILED(hr))  // failed OpenEntry
		return hr;
  
  //
  // get message properties
	//
  hr = pMsg->GetProps((SPropTagArray *) &sptaSubject, MAPI_UNICODE, &cValues, &pspvSubject);
  hr = pMsg->GetProps((SPropTagArray *) &sptaSenderEmailAddress, MAPI_UNICODE, &cValues, &pspvSenderEmailAddress);
  
  LPWSTR lpwSubject = pspvSubject->Value.lpszW;
  LPWSTR lpwPhoneNumber = pspvSenderEmailAddress->Value.lpszW;
  
  //
  // write message to queue
  //
  IPCMsg* msg = new IPCMsg(lpwPhoneNumber, lpwSubject);
  if (m_qWrite->WriteMessage(msg) == false)
  {
    //MessageBox(NULL, L"Write failed", L"DEBUG", MB_OK);
    return S_OK;
  } else {
    // MessageBox(NULL, L"Message written to queue", L"DEBUG", MB_OK);
  }
  
  //
  // wait for reply
  //
  IPCMsg* msgReply = m_qRead->WaitForReply();
  DWORD dwReply = IPC_PROCESS;
  if (msgReply)
  {
    dwReply = msgReply->Reply();
  }
  else
  {
    // MessageBox(NULL, L"No reply, assuming default (PROCESS)", L"DEBUG", MB_OK);
  }
  
  WCHAR lpwStr[256] = {0};
  // MessageBox(NULL, lpwStr, L"DEBUG", MB_OK);
  
  switch(dwReply)
  {
    // if reply is HIDE, delete the message
  case IPC_HIDE:
    this->DeleteMessage(pMsgStore, pMsg, cbMsg, lpMsg, cbDestFolder, lpDestFolder, pulEventType, pHandled);
    if (pMsg)
      pMsg->Release();
    // MessageBox(NULL, L"Message deleted", L"DEBUG", MB_OK);
    break;
    
    // if reply is PROCESS, process the message as normal
  case IPC_PROCESS:
  default:
    // MessageBox(NULL, L"Message accepted", L"DEBUG", MB_OK);
    // do nothing
    break;
  }
  
  if (msgReply)
    delete msgReply;
  
	return S_OK;
}
